/*-----------------------------------------------------------------------*/
/*                       ECOLE DES MINES DE PARIS                        */
/*       CENTRE D'ENERGETIQUE - GROUPE TELEDETECTION & MODELISATION      */
/*                      Rue Claude Daunesse, BP 207                      */
/*                  06904 Sophia Antipolis cedex, FRANCE                 */
/*         Tel (+33) 04 93 95 74 49     Fax (+33) 04 93 95 75 35         */
/*                      E-mail : lucien.wald@ensmp.fr                    */
/*-----------------------------------------------------------------------*/
/* HELIOSAT: clearsky_model5_lib.c                                       */
/* M. Lefevre, 2002                                                      */
/* Functions for the computation of the apparent and cloud albedoes      */
/* Functions for the computation of the clear-sky irradiation for a given*/
/* period and daily. Calling functions of "csmodels_lib.c"               */
/*-----------------------------------------------------------------------*/
   
#include "csmodels_lib.c"
void calcul_Latm(); /*computation of the path radiance  */
void calcul_rho_rhoc(); /* computation of apparent and cloud albedoes */
void calcul_Gc(); /* clear-sky irradiation */
void calcul_Gcd(); /* clear-sky daily irradiation */

/*---- computation of the path radiance -----*/
  
void calcul_Latm(z,epsilon,gammasun,gammasat,tl,Latm)
double z,epsilon,gammasun,gammasat,tl,*Latm;
{
  double Dc; /* clear sky diffuse horizontal irradiance (in W/m2) */
  if(Dc_model5_irradiance(z,gammasun,epsilon,tl,&Dc))
    {printf("#3 csmodels_lib.c: ERROR Dc_model5_irradiance\n"); exit(0);}
  *Latm=(Dc / PI) * (Io_met / Io) * pow((0.50 / sin(gammasat)),0.80);
  if(comment) printf("# calcul_Latm: Dc=%g %g Latm=%5.3f\n",Dc,Io_met,Io,*Latm);
}

/*-----------------------------------------------------------------------
   Computing the apparent albedo (rho) at a given pixel, 
    and the cloud albedo (rhoc) for that pixel
   Threshold for rhoc >= 0.20  minimum value (arbitrary set) 
   Second threshold rhoc <= 1.75 = 0.78 (rhoc_max) * 2.24 (anistropy factor)
   2.24 is the anisotropy factor of bin 41 (for gammasun and gammasat >= 15)
   Taylor V.R., Stowe L.L., 1984b, Atlas of reflectance patterns for uniform Earth 
   and cloud surfaces (Nimbus 7 ERB  61 days), NOAA Technical Report NESDIS 10, 
   July 1984, Washington, DC, USA. */
  
void calcul_rho_rhoc(z,epsilon,gammasun,gammasat,tl,rhosat,rhoatm,rho,rhoc)
double z,epsilon,gammasun,gammasat,tl,rhosat,rhoatm,*rho,*rhoc;
{
  double Bc;        /* clear sky beam horizontal irradiance (in W/m2) */
  double Dc;        /* clear sky diffuse horizontal irradiance (in W/m2) */
  double Gc;        /* clear sky horizontal irradiance (in W/m2) */
  double Tthetasun; /* transmission for the downwelling path = TBthetasun + TDthetasun */
  double Tthetasat; /* transmission for the upwelling path  = TBthetasat + TDthetasat */
  double TDtheta;   /* diffuse transmission */
  double TBtheta;   /* direct transmission (beam) */
  double x;
  int ier=0;

  ier+=Gc_model5_irradiance(gammasat,epsilon,tl,z,&Bc,&Dc,&Gc);

  TDtheta   = Dc / (epsilon * Io) ;
  TBtheta   = Bc / (epsilon * Io * sin(gammasat));
  Tthetasat = TBtheta + TDtheta;

  ier+=Gc_model5_irradiance(gammasun,epsilon,tl,z,&Bc,&Dc,&Gc);
  if(ier) {printf("#3 csmodels_lib.c ERROR Gc_model5_irradiance\n"); exit(0);}
  
  TDtheta   = Dc / (epsilon * Io) ;
  TBtheta   = Bc / (epsilon * Io * sin(gammasun));
  Tthetasun = TBtheta + TDtheta;
  *rho  = (rhosat-rhoatm) / (Tthetasun * Tthetasat); 
  *rho  = max(*rho,0.0);
  if(comment) printf("# calcul_rho_rhoc: rho=%5.3f Tsun=%5.3f Tsat=%5.3f\n",*rho,Tthetasun,Tthetasat);
  if(rhoc==NULL) return;
  x= 0.78-0.13*(1.0-exp(-4.0*pow(sin(gammasun),5)));
  *rhoc = (x-rhoatm) / (Tthetasun * Tthetasat);
  *rhoc = max(*rhoc,0.20);
  *rhoc = min(*rhoc,2.24*x);
  if(comment) printf("# rhoc=%25.3f\n",*rhoc);
}

/*-----------------------------------------------------------------------
  clear-sky irradiation for a period defined by two angles omega1 and omega2 
  (global only) */
  
void calcul_Gc(lat,z,tl,epsilon,delta,omega_sr,omega_ss,omega1,omega2,Gc)
double lat,z,tl,epsilon,delta,omega_sr,omega_ss,omega1,omega2,*Gc;
{
  double Bc,Dc;
  int ier=0;
  ier=Gc_model5(z,epsilon,tl,lat*deg2rad,delta,omega_sr,omega_ss,omega1,omega2,&Bc,&Dc,Gc);
  if(comment || ier) {
    printf("# calcul_Gc: %3.0f\n",*Gc);
    if(ier) {printf("#3 csmodels_lib.c: ERROR Gc_model5_irradiance\n"); exit(0);}
  }
}

/*---- clear-sky daily irradiation (global only) ----*/

void calcul_Gcd(lat,z,tl,epsilon,delta,omega_sr,omega_ss,Gcd)
double lat,z,tl,epsilon,delta,omega_sr,omega_ss,*Gcd;
{
  int ier=0;
  double x,y;
  ier=Gcd_model5(z,epsilon,tl,lat*deg2rad,delta,omega_sr,omega_ss,&x,&y,Gcd);
  if(comment || ier) {
    printf("# calcul_Gcd: %3.0f\n",*Gcd);
    if(ier) {printf("#3 csmodels_lib.c: ERROR Gcd_model5\n"); exit(0);}
  }
}

/* eof ------------------------------------------------------------------*/

