/*-------------------------------------------------------------------------*/
/*                        ECOLE DES MINES DE PARIS                         */
/*        CENTRE D'ENERGETIQUE - GROUPE TELEDETECTION & MODELISATION       */
/*                       Rue Claude Daunesse, BP 207                       */
/*                   06904 Sophia Antipolis cedex, FRANCE                  */
/*          Tel (+33) 04 93 95 74 49     Fax (+33) 04 93 95 75 35          */
/*                       E-mail : lucien.wald@ensmp.fr                     */
/*-------------------------------------------------------------------------*/
/* HELIOSAT: glitter_lib.c                                                 */
/* M. Lefevre, 2002                                                        */
/* Functions to detect the ocean glitter area and to correct cloud index   */
/*-------------------------------------------------------------------------*/

/* To remove the glitter effect, a window is defined centred on the specular
   point. This point is defined as the pixel of the image for which the 
   viewing and sun zenithal angles are the closest (thetasat=thetasun) and 
   the azimuths difference phisat-phisun=180. The search is performed 
   between the latitudes 30 N and 30 S approximately. thetasat and 
   thetasun should be greater than 40. Otherwise, no correction is applied.
   The window 1 is approximately 2000 km in radius. Only are considered the
   pixels known as belonging to the ocean. 
   If Kch>0.8 for the current pixel, the sky is clear. The window 2 of 3x3
   pixels is defined centred on the current pixel. If the number of pixels
   exhibiting cloud indices<0.2, is greater than 5 (60%), the pixel is said 
   cloud-free and the cloud index is set to 0.
---------------------------------------------------------------------------*/

int find_glitter();
void remove_glitter();
void scan_glitter();


/*-------------------------------------------------------------------------*/
int find_glitter(yyyy,ddd,slot,l,c)
int yyyy,ddd,slot,*l,*c;
{      
  double lat,lon,z,day_angle,delta,epsilon,omega_sr,omega_ss,gammasat,phisat,gammasun_noon,gammasun,omega1,omega2,phisun,dphi,dgamma,tst,min=999,*tlam2=(double *) malloc(sizeof(double)*14);
  int lig,col,lig0=0,col0=0,pix,Id;
/* there are some functions specific to the Ecole des Mines implementation 
   of Heliosat-II.
   Get_IdB2 provides the pixel identifier, given line (row) and column 
   Get_info_Id provides latitude, longitude, TL and z for this pixel
   GetParam_day or _hour provide angles */

  /* window 1: searching the pixel, center of the specular area, by scanning
     the image, line after line (lig) and column after column (col)*/
  for(lig=g_lig[0] ; lig<=g_lig[1]; lig++)
    for(col=g_col[0] ; col<=g_col[1] ; col++) {
      if(Get_IdB2(lig,col,&Id,&pix)==0) continue;
      Get_info_Id(Id,NULL,NULL,&lat,&lon,tlam2,&z);
      if(z>0) continue; /* pixel terre */
      GetParam_day(yyyy,ddd,lat,lon,&day_angle,&delta,&epsilon,&omega_sr,&omega_ss,&gammasat,&phisat,&gammasun_noon);
      GetParam_hour(slot,lig,lat,lon,day_angle,delta,&gammasun,&omega1,&omega2,&phisun,&tst);
      if((gammasun*=rad2deg)<50.0) continue;/* elevation should be greater than 50 */
      if((gammasat*=rad2deg)<50.0) continue;
      phisat*=rad2deg; phisun*=rad2deg;
      if(lat>0) dphi=fabs(phisat-180.0-phisun); else dphi=fabs(phisat-180.0+phisun);
      if(fabs(dphi-180.0) > 0.5) continue; /* searching delta_azimuth=180 */
      dgamma=fabs(gammasat-gammasun);
      if(dgamma > 10.0) continue; /* searching gammasun=+/-gammasat */
      if(dgamma < min) {
	min=dgamma; lig0=lig; col0=col;
	if(comment) printf("# lat %3.0f lon %3.0f gamsat %5.2f sun %5.2f dgam=%5.3f phisat %3.0f sun %3.0f dphi %3.3f lig %d col %d\n",lat,lon,gammasat,gammasun,dgamma,phisat,phisun,dphi,lig0,col0);
      }
    }
  free(tlam2);
  *l=lig0; *c=col0;
  if(lig0*col0) printf("glitter lig %d col %3d ",lig0,col0);
  return lig0*col0;
}

/*-------------------------------------------------------------------------*/

void remove_glitter(lig0,col0,cn)
int lig0,col0;
unsigned char *cn;
{
  double lat,lon,z,*tlam2=(double *) malloc(sizeof(double)*14);
  int lig,col,pix,pix0,Id,i,j,n;

  /* lig0 and col0 are the central row and column of the specular area.
     cn is the corrected cloud index*/
  for(lig=max(g_lig[0],lig0-r) ; lig < lig0+r ; lig++) {
    for(col=col0-r ; col < col0+r ; col++) {
      if(Get_IdB2(lig,col,&Id,&pix0)==0) continue;
      Get_info_Id(Id,NULL,NULL,&lat,&lon,tlam2,&z);
      if(z>0) continue;/* if pixel belongs to land mass, do not proceed */
/* cn+pix0 is the cloud index for the central pixel pix0 */
      if(*(cn+pix0) >= n70) continue;/* cloud index>=0.2 => cloudy */
      /* window 2 is 3*3 pixels*/
      for(n=0,i=-1;i<=1;i++)
	for(j=-1;j<=1;j++) {
	  pix=(lig+i-1)*B2_NBCOL+col+j-1;
	  if(*(cn+pix) < n70) n++;
	}
      if(n>5) *(cn+pix0)=n50;/* if ~60% of pixels are clear_sky, cloud index=0 */
    }
  }
  free(tlam2);
}

/*-------------------------------------------------------------------------*/
void scan_glitter(yyyy,ddd1,ddd2)
int yyyy,ddd1,ddd2;
{
/* a function specific to the Ecole des Mines implementation of Heliosat-II*/
/* in a given directory, searching for the data exhibiting glitter area */
  int slot=0,ddd,mm,lig,col;
  for(ddd=ddd1 ; ddd<=ddd2 ; ddd++) {
    yyddd2mm(yyyy,ddd,&mm);
    for(slot=g_slot[0] ; slot<=g_slot[1] ; slot++) {
      printf("\n%02d/%04d   day %3d  slot %d ",mm,yyyy,ddd,slot);
      find_glitter(yyyy,ddd,slot,&lig,&col);
    }
  }
}
/* eof --------------------------------------------------------------------*/
