/*-----------------------------------------------------------------------*/
/*                       ECOLE DES MINES DE PARIS                        */
/*       CENTRE D'ENERGETIQUE - GROUPE TELEDETECTION & MODELISATION      */
/*                      Rue Claude Daunesse, BP 207                      */
/*                  06904 Sophia Antipolis cedex, FRANCE                 */
/*         Tel (+33) 04 93 95 74 49     Fax (+33) 04 93 95 75 35         */
/*                      E-mail : lucien.wald@ensmp.fr                    */
/*-----------------------------------------------------------------------*/
/* helio.h     Mireille Lefevre  2002                                    */
/* Description of the global variables for the method Heliosat-II        */
/*-----------------------------------------------------------------------*/

#include <stdio.h>    /* I/O standard library */
#include <string.h> 
#include <math.h> 
/* #include <request.h> */

/* define functions maximum and minimum */
#define max(x,y) ((x)>(y) ? (x) : (y))  
#define min(x,y) ((x)<(y) ? (x) : (y))
/* define function swapint to swap bytes into an integer */
#define swapint(mot) ((mot & 0X000000FF) << 24) + ((mot & 0X0000FF00) << 8) +((mot & 0X00FF0000) >> 8) + ((mot & 0XFF000000) >> 24);

/*-----------------------------------------------------------------------*/
/*-- directories --------------------------------------------------------*/

#define im_albedo      "../albedo/im_albedo.bin" /* reference rhog */
#define im_TLinke      "../linke" /* images TL5_mois.bin */ 
#define im_TB5         "../elevation" /* image TerrainBase tbas_17.bin */ 
#define im_HR          "../../meteosat/HR_images/VIS"
#define im_B2          "../B2_vis" /* ../../meteosat/B2_images" /* images meteosat */
#define im_IdB2        "../info/Id_imgB2.bin" /* identifiers pixels */
#define tab_IdB2       "../info/tab_idB2.bin" /* information about identified pixels */
#define tab_coeffcalib "../info/tab_coeffcalib"/* calibration coeff. */
#define tab_data_wmo   "../../ground measurements from WMO "
#define tab_wmo        "../info/list_wmo"
#define tab_spectral   "../info/spectral.txt"
#define im_cloud_index "../dbcloud" /* temporary images of albedos gYYMM*bin and cloud index nYYDDDslot.bin before their integration in the database */
#define db_cloud_index "../dbcloud" /* database of the cloud index db_YYYY.bin */

/*-----------------------------------------------------------------------*/
/*-- meteosat 5 VIS -----------------------------------------------------*/

#define OFFSETlig  42 /* line number of the NW corner of the sub-image HR*/
#define OFFSETcol 850 /* same but for column*/
#define HR_NBLIG  456 /*number of lines in the sub-image HR (resol. 5 km)*/
#define HR_NBCOL  938 /* number of columns*/
#define B2_NBLIG  416 /* number of lines in the image B2 (low resolution)*/
#define B2_NBCOL  416 /* number of columns */
#define NB_SLOT    48 /* number of slots during a day */
#define NB_ID  118500 /* number of identified pixels (that can be used): usable=1 */
#define NBPIX9      9 /* should be square of 1 3 5 or 7. For spatial interpolation */

double UT[49]={0,0.0,0.50,1.0,1.50,2.0,2.50,3.0,3.50,4.0,4.50,5.0,5.50,6.0,6.50,7.0,7.50,8.0,8.50,9.0,9.50,10.0,10.50,11.0,11.50,12.0,12.50,13.0,13.50,14.0,14.50,15.0,15.50,16.0,16.50,17.0,17.50,18.0,18.50,19.0,19.50,20.0,20.50,21.0,21.50,22.0,22.50,23.0,23.50};
 /* UT = universal time. UT[49] is the beginning time of acquisition (Southernmost line). */
 /* slot 1=0h-0h30  slot 11=5h-5h30 slot 24=11h30-12h  cf. util_TU() */

const int g_slot[2]={19, 42};  /* 1st and last slot exhibiting sun glitter */
const int g_lig[2]={170,270};  /* extreme lines of the 1st window for searching the specular center*/
const int g_col[2]={ 30,265};  /* extreme columns */
const int r  =75; /* window 2: radius (in pixels) */

/*-----------------------------------------------------------------------*/
/*-- conversion factors, data filtering and other thresholds ------------*/

#define flag_Gwmo  10     /* filter: ground measurements Gwmo >= 10 W/m2 */
#define flag_sat 15.0     /* filter: satellite elevation angle >= 15 */
#define flag_sun 15.0     /* filter: solar elevation angle >= 15 (cf. bin 41 of Taylor & Stowe, 1984) */
double  flag_sunoon;      /* =f(gammasun_noon) for computation of rhog */
const double convert_rhog=100.0; /* conversion factor for coding rhog */
const double convert_n[2]={100.0,50.0}; /* conversion factors for coding cloud index[-0.5:1.5]=cn[0:200]*/
                                        /* Note: changing these values affects n50 et n70 in glitter_lib.c */
const unsigned char n50=50;          /* cloud index=0 after conversion */
const unsigned char n70=70;          /* cloud index=0.2 after conversion */
const int nbdata_min_wmo[2]={6,12};  /* minimum number of available hourly data per day */
const int nbdata_min_slotB2[2]={2,3};/* threshold=f(season)=f(gammasun_noon)*/
const int nbdata_min_slotHR[2]={6,9};/*threshold for Gh ground and images*/
const float validite=0.60;           /* 60% of valid data for computing means and sums */
/*-----------------------------------------------------------------------*/
/*-- database of cloud index  -------------------------------------------*/

const int nbval_day=8;     /* number of values per day = nb slots provided by Eumetsat as a standard */
const int size_val=2;      /* number of bytes per value to be stored 1:slot  2:cloud index */
const int nbdays=366;      /* number of days per pixel in a year */
const int size_enr=5856;   /* siez in bytes of one yearly time-series (record) (without identifier) 2*8*366 */
const int size_entete=128; /* numbers of bytes of the first record (header) */
const int rejet=254;       /* valid data are less than this value */
const int pos_slot[49]={-1,-1,-1,0,0,0,0,0,-1,1,1,1,1,1,-1,2,2,2,2,2,-1,3,3,3,3,3,-1,4,4,4,4,4,-1,5,5,5,5,5,-1,6,6,6,6,6,-1,7,7,7,7};
/* rank of each slot in a record */

/*-----------------------------------------------------------------------*/
/*-- Photosynthetically active radiation PAR ----------------------------*/
#define nb_lambda 70      /* number of wavelength in the spectra table */
#define Zmax 300          /* max. depth */
const int lambda0=310;    /* 1st wavelength in tab_spectral: 310nm  */
const int lambda_step=10; /* each wavelength = 10nm */
const int par1=400;       /* lower wavelength for PAR */
const int par2=700;       /* upper .................. */
const double N=1.340;     /* refraction index of seawater */
int Z;  
/*-----------------------------------------------------------------------*/
/*-- global variables ---------------------------------------------------*/

#define web 0             /* To make the program working or not through the Web. See helio_Gd */
#define PI 3.141592654
#define R  6371.0         /* Earth radius in km */
#define Io 1367.0         /* solar constant in W/m2 */
#define deg2rad 0.0174533 /* decimal degrees to radian 90=1.57rd */
#define rad2deg 57.29578  /* converts radians into degrees */
const int nday[13] ={0,31,28,31, 30, 31, 30, 31, 31, 30, 31, 30, 31}; /* nb of days per month */
const int Sday[13] ={0,31,59,90,120,151,181,212,243,273,304,334,365}; /* 1st julian day -1 for each month */
const int Sdayb[13]={0,31,60,91,121,152,182,213,244,274,305,335,366}; /* same but for leap year */
const char *mois[13]={"","jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"};

double Io_met;             /* solar constant=f(t) W/m2 for the spectral window of Meteosat*/

/* Default values of some variables to execute the program that are passed as arguments on the command line */
int comment=0;             /* arg -c       if comment, then verbose */
int arg_id=0;              /*     -id #     */
char *date;                /*     -d yyyymm */
int date1,date2;           /*     -d yyyymm           */
int arg_slot_impair_only=1;/*     -slot pour intgrer slots pairs (HR) */
int afrique=1, europe=1;   /*     -europe -afrique    */
char *arg_wmo;             /*     -wmo                */
double test=0;             /*     -test */
int HR=0,B2=1,HRB2=0;      /* by default B2 is processed */
int NBPIX=9;               /* by default interpolation for Gh is performed on 9 pixels */

/*-- eof ----------------------------------------------------------------*/
