/*-------------------------------------------------------------------------*/
/*                        ECOLE DES MINES DE PARIS                         */
/*        CENTRE D'ENERGETIQUE - GROUPE TELEDETECTION & MODELISATION       */
/*                       Rue Claude Daunesse, BP 207                       */
/*                   06904 Sophia Antipolis cedex, FRANCE                  */
/*          Tel (+33) 04 93 95 74 49     Fax (+33) 04 93 95 75 35          */
/*                       E-mail : lucien.wald@ensmp.fr                     */
/*-------------------------------------------------------------------------*/
/* map_cloud.c    Mireille Lefevre  2002                                   */
/* Software for the crea&tion of an image (raw format) of cloud index for  */
/* given month.                                                            */
/* Only pixels for which gammasat >= flag_sat (=pixels usables) and        */
/* gammasun >= flag_sun (bin 41 de Taylor & Stowe, 1984) are processed.    */
/* Cloud indices n are converted in integers (cn) between 0 and 200,       */
/* cn=n*100+50, 254 denotes missing values, 255 not usable pixels          */
/*-------------------------------------------------------------------------*/
/* Output : nYYDDDslot.bin, where YY is year, DDD julian day
  image raw of cloud index format B2: temporary image deleted after 
  integration into the cloud index database db_yyyy*

  if arg -id display the cn value for a specified pixel. Note: in this case, 
  the routine glitter() is not executed.
---------------------------------------------------------------------------*/
/* arguments : -d YYYYMM ou YYYYMMDD (year+month+day)
              [-id pixel identifier from 1 to 118500]
---------------------------------------------------------------------------*/
#include "helio.h"
#include "spatio_time_lib.c"
#include "clearsky_model5_lib.c" 
#include "map_lib.c"
#include "glitter_lib.c"

void parse_args();
void convert();
void print_output();


main(argc, argv)
int argc;
char *argv[];
{
  int size = sizeof(double) * B2_NBLIG*B2_NBCOL;
  unsigned char *cn=NULL;
/* malloc is a C function for the dynamical allocation of memory */
  double *Lsat=(double *) malloc(size);/* radiance at satellite level */
  double  Latm;                        /* path radiance */  
  double *rhog=(double *) malloc(size);/* ground albedo */
  double  rhosat;       /* reflectance at satellite level */
  double  rhoatm;       /* atmosphere reflectance (path reflectance)*/ 
  double  rho;          /* reflectance corrected for the atmospheric effects*/
  double  rhoc;         /* cloud albedo */
  double *cloud=(double *) malloc(size);/* cloud index */
  double  lat,lon;      /* latitude  longitude */
  int     lig,col;      /* row column of the Meteosat pixel */
  double  z;            /* altitudes (m) */
  double  gammasat;     /* sat. elevation above horizon (radians) */
  double  gammasun;     /* solar elevation angle (radians) */
  double  gammasun_noon;/* idem at noon  */
  double  phisun;       /* solar azimuth angle (radians) */
  double  phisat;       /* satellite azimuth (radians) */
  double  omega_sr;     /* sunrise solar hour angle (radians) */
  double  omega_ss;     /* sunset solar hour angle (radians) */
  double  omega1;       /* UT-0.5 solar hour angle (radians) */
  double  omega2;       /* UT+0.5 solar hour angle (radians) */
  double  delta;        /* declination angle at noon (radians) */
  double  day_angle;    /* */
  double  epsilon;      /* eccentricity of orbit */
  double  tst;          /* true solar time */
  int yy,yyyy,mm=0;     /* year month day slot  */
  int dd=0,dd1=0,dd2=0;
  int ddd=0,ddd1=0,ddd2=0;
  int slot=0,slot1=1,slot2=NB_SLOT;
  double mu,offset;     /* calibration coeff. */
  unsigned char dark;   /* idem */
  double tl;            /* Linke turbidity factor TL for this day */
  double *tlam2=(double *) malloc(sizeof(double)*14);  /* TL Jan to Dec with repetition of extremes for interpolation */
  int pix,Id;           /* B2 pixel identifier */
  int i,m=0,lig1,lig2,col1,col2;

  B2=1; HR=HRB2=0;
  parse_args(argc,argv);  /* parsing arguments. Constructing date from the name of the file */
  if(strlen(date)==4)sscanf(date,"%04d",&yyyy);
  if(strlen(date)==6)sscanf(date,"%04d%02d",&yyyy,&mm);
  if(strlen(date)==7)sscanf(date,"%04d%03d",&yyyy,&ddd);
  if(strlen(date)==8)sscanf(date,"%04d%02d%02d",&yyyy,&mm,&dd);
  if(strlen(date)==9)sscanf(date,"%04d%03d%02d",&yyyy,&ddd,&slot);
  yy= yyyy % 100;
  if(mm!=0) {
    if(dd==0) {dd1=1; dd2=31;} else dd1=dd2=dd;
    while(ddd1==0) ddmmyy2ddd(dd1--,mm,yyyy,&ddd1);
    while(ddd2==0) ddmmyy2ddd(dd2--,mm,yyyy,&ddd2);
    dd=dd1+1;
  }
  else {
    if(ddd==0) {ddd1=1; ddmmyy2ddd(31,12,yyyy,&ddd2);}
    else ddd1=ddd2=ddd;
  } 
  if(slot!=0) slot1=slot2=slot;
  if(arg_id){
    Get_info_Id(arg_id,&lig1,&col1,NULL,NULL,NULL,NULL);
    lig2=lig1; col2=col1;
    }
  else {
    lig1=1; lig2=B2_NBLIG;  /* lig means line or row, col means column */
    col1=1; col2=B2_NBCOL;  /* sizes of image = NBLIG rows and NBCOL columns */
  }
  
  printf("# map cloud index %d day %d  %d\n",yyyy,ddd1,ddd2);
  for(ddd=ddd1 ; ddd<=ddd2 ; ddd++){
    if(yyddd2mm(yyyy,ddd,&mm)!=m) {m=mm; Get_rhog(yy,mm,rhog);}  /* get rhog */
    GetCoeff_calib(yyyy,ddd,&dark,&mu,&offset); /* get calibration coeffs */
    for(slot=slot1; slot<=slot2 ; slot++){
      if(!loadminimage(yy,mm,ddd,slot,&cn)) continue; /* read Meteosat image */
      calibration(cn,dark,mu,offset,Lsat); /* perform calibration for whole image */
      for(i=0; i<B2_NBLIG*B2_NBCOL ; i++) *(cloud+i)=999.0;
      for(lig=lig1; lig<=lig2 ; lig++){
 	for(col=col1 ; col<=col2 ; col++) {
	  if(Get_IdB2(lig,col,&Id,&pix)==0) {*(cloud+pix)=255.0; continue;}/* not usable */
	  Get_info_Id(Id,NULL,NULL,&lat,&lon,tlam2,&z); /* get lattitude, longitude, TL and z for this pixel */
	  GetParam_day(yyyy,ddd,lat,lon,&day_angle,&delta,&epsilon,&omega_sr,&omega_ss,&gammasat,&phisat,&gammasun_noon);
	  GetParam_hour(slot,lig,lat,lon,day_angle,delta,&gammasun,&omega1,&omega2,&phisun,&tst);
	  if(gammasun*rad2deg < flag_sun) continue; /* do not process this pixel if the sun is too low */
	  interpol_TL(ddd,mm,tlam2,&tl);  /* compute TL for the day by interpolation of monthly values */
	  calcul_albedo(*(Lsat+pix),epsilon,gammasun,&rhosat); /* compute reflectance rhosat */
	  calcul_Latm(z,epsilon,gammasun,gammasat,tl,&Latm); /* compute path radiance */
	  calcul_albedo(Latm,epsilon,gammasun,&rhoatm); 
	  calcul_rho_rhoc(z,epsilon,gammasun,gammasat,tl,rhosat,rhoatm,&rho,&rhoc);
	  calcul_n(rho,*(rhog+pix),rhoc,cloud+pix);/* cloud index */
	  if(comment) getchar();
	}
      }
      if(arg_id) convert(cloud+pix,cn);
      else {
	convert(cloud,cn);
	if(find_glitter(yyyy,ddd,slot,&lig,&col)) remove_glitter(lig,col,cn);
      }
      print_output(cn,yy,ddd,slot);
    }
  }
  printf("#0 end\n");
}

/*-------------------------------------------------------------------------*/
void convert(cloud,cn)
double *cloud;
unsigned char *cn;
{
  int i;
  if(arg_id){
    if(*cloud==255.0)      *cn=(unsigned char) 255;/* pixel not usable */
    else if(*cloud==999.0) *cn=(unsigned char) 254;/* missing value */
    else                   *cn=(unsigned char) floor(*cloud*convert_n[0] + convert_n[1]+0.50);
    return;
  }
  for(i=0 ; i<B2_NBLIG*B2_NBCOL ; i++){
    if(*(cloud+i)==255.0)      *(cn+i)=(unsigned char) 255;/* pixel not usable */
    else if(*(cloud+i)==999.0) *(cn+i)=(unsigned char) 254;/* missing value */
    else                       *(cn+i)=(unsigned char) floor(*(cloud+i)*convert_n[0] + convert_n[1]+0.50);
  }
}

/*-------------------------------------------------------------------------*/
void print_output(cn,yy,ddd,slot)
unsigned char *cn;
int yy,ddd,slot;
{
  static FILE *fo=NULL;
  static char *fic=NULL;
  if(fic==NULL) fic=(char *) malloc(80);

  if(arg_id) {
    printf("%02d%03d slot %2d cn= %3d ",yy,ddd,slot,*cn);
    if(*cn<254) printf("cloud=%3.1f",((float)*cn-convert_n[1])/convert_n[0]);
    printf("\n");
  }
  else {
    sprintf(fic,"%s/%02d/n%02d%03d%02d.bin",im_cloud_index,yy,yy,ddd,slot);
    printf("# output image %s\n",fic);
    fo=fopen(fic,"w");
    if(fwrite(cn,B2_NBLIG*B2_NBCOL,1,fo)!=1) printf("#3 ERROR criture %s\n",fic);
    fclose(fo);
  }
}

/*-------------------------------------------------------------------------*/
void parse_args(argc,argv)
int argc;
char *argv[];
{
  int i=1;
  date=(char *) malloc(10);
  while(i<argc) {
    if(strncmp(argv[i],"-c",2)==0) comment=1;
    else if(strncmp(argv[i],"-d",2)==0) sscanf(argv[++i],"%s",date);
    else if(strncmp(argv[i],"-id",3)==0) sscanf(argv[++i],"%d",&arg_id);
    i++;
  }
  if(atoi(date)<198501) {printf("give a date :  -d yyyymm\n"); exit(0);}
  printf("\n\n");
}
/* eof --------------------------------------------------------------------*/
