/*-------------------------------------------------------------------------*/
/*                        ECOLE DES MINES DE PARIS                         */
/*        CENTRE D'ENERGETIQUE - GROUPE TELEDETECTION & MODELISATION       */
/*                       Rue Claude Daunesse, BP 207                       */
/*                   06904 Sophia Antipolis cedex, FRANCE                  */
/*          Tel (+33) 04 93 95 74 49     Fax (+33) 04 93 95 75 35          */
/*                       E-mail : lucien.wald@ensmp.fr                     */
/*-------------------------------------------------------------------------*/
/* map_lib.c    M. Lefevre   2001                                          */
/* Several routines for the calibration, computation of the reflectance,   */
/* computation of the clear-sky index, hourly irradiation, clearness index */
/* and daily irradiation.                                                  */
/*-------------------------------------------------------------------------*/

/*                                                                         */
/* HELIOSAT: map_lib.c    2002                                                                           

---------------------------------------------------------------------------*/
void calibration();
void calcul_albedo();
void calcul_n();
void calcul_Kc();
void correction_Kc();
void calcul_Gh();
void calcul_Gd();
void calcul_Kt();
void GetCoeff_calib();
void GetCoeff_calib_old();
void Get_rhog();
int  loadminimage();
int  min_met();

/*---------------------------------------------------------------------------
   Converting the Meteosat digital count (cn) in radiance (Lsat)         */

void calibration(cn,dark,mu,offset,Lsat)
unsigned char *cn;
unsigned char dark;
double mu,offset,*Lsat;
{
  static int nbpix=0;
  int i;
  if(nbpix==0) if(B2) nbpix=B2_NBLIG*B2_NBCOL; else nbpix=HR_NBLIG*HR_NBCOL;
  for(i=0; i<nbpix ; i++)  /* proceed for all pixels of the image of size NBLIG * NBCOL */
   *(Lsat+i) = mu * (double)(*(cn+i)-dark) + offset;



  if(comment) {
    i=nbpix/2;
    printf("# Calibration: dark=%d mu=%5.3f offset=%5.3f pix.%d: cn=%d Lsat=%5.3f\n",dark,mu,offset,i,*(cn+i),*(Lsat+i));
  }
}

/*---------------------------------------------------------------------------
   Computation of a reflectance (rho) from a radiance (L)                  */

void calcul_albedo(L,epsilon,gammasun,rho)
double L,epsilon,gammasun,*rho;
{
  *rho =  PI * L / (Io_met * epsilon * sin(gammasun));
  *rho = max(*rho,0.0);
  if(comment) printf("# calcul_albedo: L=%5.3f  rho = %5.3f\n",L,*rho);
}

/*-------------------------------------------------------------------------
  computation of the cloud index (cloud), limited in [-0.5:1.5]          */

void calcul_n(rho,rhog,rhoc,cloud)
double rho,rhog,rhoc,*cloud;
{
  if(1) { /* pour l'instant on garde l'ancienne version */
    if(rho < 0.01)                  *cloud=0;
    else if(fabs(rho-rhog) < 0.010) *cloud=0.0;
    else if((rhoc-rhog) <=0.10)     *cloud=1.20;
    else                            *cloud=(rho-rhog) / (rhoc-rhog);
  }
  if(0) { /* modif a confirmer, utilise pour tester 1985-1990 */
    if(rhoc-rhog<=0.010)        *cloud=1.50;/* valeur arbitrairement forte */
    else                        *cloud=(rho-rhog) / (rhoc-rhog);
  }
  if(*cloud<-0.50) *cloud=-0.50;
  if(*cloud>1.50) *cloud=1.50;
  if(comment) printf("# calcul_n:        rhog=%5.2f  n=%g\n",rhog,*cloud);
}

/*-- C A L C U L _ K c  ---------------------------------------------------*/
/* Computation of the clear-sky index Kc [0.05:1.2] using the cloud index  */
/* Rigollier C., Wald L., 1999. Towards operational mapping of solar 
   radiation from Meteosat images. In Proceedings, EARSeL Symposium 
   1998 operational remote sensing for sustainable development, 
   Enschede, The Netherlands, Nieuwenhuis G., Vaughan R., Molenaar 
   M. eds, Balkema, Rotterdam, pp 385-391.                         */

void calcul_Kc(cloud,Kc)
double cloud,*Kc;
{
  if(     cloud <-0.20) *Kc = 1.20;
  else if(cloud < 0.80) *Kc = 1.0 - cloud;
  else if(cloud < 1.10) *Kc = 2.0667 - 3.6667 * cloud + 1.6667 * pow(cloud,2.0);
  else                  *Kc = 0.05;
  if(comment) printf("# calcul_Kc: %5.3f\n",*Kc);
}

/*-- C O R R E C T I O N _ K c  ---- NOT USED ------------------------*/
/* correction of the relationship Rigollier, Wald  Kc=f(cloud)             */
void correction_Kc(Kc,tst)
double *Kc,tst;
{
  *Kc-=(8.0 * tst - 104.0)/100.0 * *Kc; 
  *Kc=min(*Kc,1.20);  *Kc=max(*Kc,0.050); 
  if(comment) printf("# correction_Kc: Kc=%5.3f\n",*Kc);
}

/*-------------------------------------------------------------------------
  computation of the hourly irradiation and daily sum                    */

void calcul_Gh(Kc,Gc,Gh,Kd,Gc0)
double Kc,Gc,*Gh,*Kd,*Gc0;
{
  *Gh = Kc * Gc;
  *Gc0+=Gc;  /* summing the hourly Gc for the weighting of Kd */
  *Kd+=*Gh;  /* daily clearsky */
  if(comment) printf("# calcul_Gh: Gh= %3.0f Gh+=%5.0f\n",*Gh,*Kd);
}

/*-------------------------------------------------------------------------
  Computation of daily irradiation  */

void calcul_Gd(Gc0,Gcd,Kd,Gd)
double Gc0,Gcd,*Kd,*Gd;
{
  if(Gc0==0.0) {*Gd=-9999; printf("# Gc0=0\n");}
  else {
    *Kd/=Gc0;
    *Gd= Gcd * *Kd;
  }
  if(comment) printf("# calcul_Gd:  Kd=%4.0f  Gd=%4.0f\n",*Kd,*Gd);
}
/*---------------------------------------------------------------------------
  Computation of Kt, clearness index */

void calcul_Kt(Gd,lat,epsilon,delta,Kt)
double Gd,lat,epsilon,delta,*Kt;
{
  static int iers=0;
  int ier;
  double G0d; /* daily extraterrestrial solar irradiation (in Wh/m2) */
  *Kt=-9.99;
  if((ier=G0_day(lat*deg2rad,epsilon,delta,&G0d))==0) *Kt=Gd/G0d;
  if((iers+=ier)==1) printf("#3 calcul_Kt: ERROR\n");
  if(comment) printf("# calcul_Kt:  GOd=%4.0f  Kt=%5.3f\n",G0d,*Kt);
}

/*-------------------------------------------------------------------------*/
/* Reading the calibration coeffs file before 1998
   format :
   1: day from 581 for 1/1/85
   2: year
   3: julian_day
   4: slot. if slot=1 no change.
            If slot=n the coeffs are valid for slots >= n
                      for slots < n take coeff. of day before
   5: dt  if dt!=0 image not available => interpolation between 2 avail. images                                      spares par dt=nb de jours
   6: Io_met solar constant for the sensor
   7: calibration coeff.: mu
   8:                     offset
   9:                     CN dark
*/

/*Reading calibration coefficients provided by Eumetsat, for period after January 1998 (included)*/
/*Eumetsat provides 2 values instead of 3 in HelioClim: mu and space_count (expressed here by the variable 'dark'), offset is set to zero*/
/*Sylvain CROS october 2003*/

void GetCoeff_calib(yyyy0,ddd0,dark0,mu0,offset0)
int yyyy0,ddd0;
unsigned char *dark0;
double *mu0,*offset0;
{
  int yyyy,dark,slot,dt,ok=0,dddlaunch;
  static int ddd;
  double mu,offset;
  
  *dark0=0;
  *mu0=*offset0=0.0;

  if ((yyyy0==1998) && (ddd0<154))    /*Coef constant for Meteosat-6 till 3 June 1998 (yyyy=1998 jjj=153)*/
   {mu=0.8600;
    dark=5;
    offset=-(mu*0.6);               /*Adjustment in order to keep 'dark' as an integer*/
    Io_met=692.16;

   }
      else

	
	{ dddlaunch=(yyyy0-1998)*365+(ddd0-154);if (yyyy0>1999 && ddd0>60) dddlaunch++;

    mu=0.926+6.63411*dddlaunch*0.00001;       /*For Meteosat-7 */ 
    dark=5;                          /* mu and dark computed by Govaert et al. 2001, linear formula */ 
    offset=0;                         /*dddlaunch: nb days between 3/6/1998 and current day*/
  Io_met=691.6;                       /*0.926: mu at 3/6/1998; 6.63411: daily drift(Wm-2sr-1/day)*/
}                                     /* processing of leap years */
 
 
  *dark0=(unsigned char)dark;
  *mu0=mu;
  *offset0=offset;
}

/*Lecture des coefficients de calibration  partir de la base de donnes (www.helioClim.net) pour la priode de 1985  1997*/
/* Fonction non appele dans cette prsente version*/
void GetCoeff_calib_old(yyyy0,ddd0,dark0,mu0,offset0)
int yyyy0,ddd0;
unsigned char *dark0;
double *mu0,*offset0;
{
  int yyyy,dark,slot,dt,ok=0;
  static int ddd;
  double mu,offset;
  static char *buf=NULL;
  static FILE *fi=0;
  if(fi==0) {
    if((fi=OpenFile(tab_coeffcalib))==0) exit(0);
    buf=(char *) malloc (200);
  }
  if(ddd0<ddd) rewind(fi);
  *dark0=0;
  *mu0=*offset0=0.0;
  while(fgets(buf,200,fi)){
    sscanf(buf,"%*d;%d;%d;%d;%d;%lf;%lf;%lf;%d",&yyyy,&ddd,&slot,&dt,&Io_met,&mu,&offset,&dark);
    if(ddd==ddd0 && yyyy==yyyy0) {ok=1; break;}
  }
  if(ok==0) {printf("#3 pas de coefficients de calibration pour day %d %d\n",yyyy0,ddd0); exit(0);}
  if(slot!=1) {ok=0;if(comment)printf("# coeff. calibration changement de slot: ");}
  if(dt!=0) {ok=0;if(comment)printf("# coeff. calibration interpols: ");}
  if(comment && ok==0) printf("# %d d.%d dt=%d slot=%d mu=%5.3f offset=%5.3f dark=%d Io_met=%g\n",yyyy,ddd,dt,slot,mu,offset,dark,Io_met);
  *dark0=(unsigned char)dark;
  *mu0=mu;
  *offset0=offset;
}

/*---------------------------------------------------------------------------
  Reading albedos (rhog minimum) in B2 images without header
  created with map_rhog.c and map_rhog_ref.c
  if yy and mm =0 take the image of reference albedo.
  if yy and mm are valid, take the constructed maps for rhog 
*/
void Get_rhog(yy,mm,rhog)
int yy,mm;
double *rhog;
{
  char *fic=(char *) malloc(100);
  unsigned char *cn=(unsigned char *) malloc(B2_NBLIG*B2_NBCOL);
  FILE *fi=0;
  int i;
  static int sav_mm=-1;
  if(mm==sav_mm) return;/* dj charg */
  if(yy+mm==0) /* rhog_ref */
    sprintf(fic,"%s\0",im_albedo);
  else /* rhog mensuels */
    sprintf(fic,"%s/%02d/g%02d%02d.bin\0",im_cloud_index,yy,yy,mm);
  if((fi=OpenFile(fic))==0) exit(0);
  sav_mm=mm;
  if(fread(cn,1,B2_NBLIG*B2_NBCOL,fi)!=B2_NBLIG*B2_NBCOL)
    {printf("#3 Get_rhog ERROR lecture %s\n",fic); exit(0);}
  for(i=0 ; i<B2_NBLIG*B2_NBCOL ; i++) {
    if(*(cn+i)==255) *(rhog+i)=255.0; /* pixel non usable */
    else             *(rhog+i)=(double)*(cn+i)/convert_rhog;
  }
  fclose(fi);
  free(fic);free(cn);
}

/*-------------------------------------------------------------------------*/
int loadminimage(yy,mm,ddd,slot,cni)
int yy,mm,ddd,slot;
unsigned char **cni;
{
  static unsigned char *cn=NULL;
  static FILE *fi=0;
  static unsigned char *buf=NULL;
  static char *path=NULL;
  static char *fic=NULL;
  static int buflen=0,nblig=0,nbcol=0;
  int i,ier=0;
  if(buf==NULL) {
    path=(char *) malloc(30);
    if(B2) {
      nblig=B2_NBLIG;
      nbcol=B2_NBCOL;
      buflen=512;
      sprintf(path,"%s",im_B2);
      printf("Chemin: %s\n",im_B2);
    }
    else {
      nblig=HR_NBLIG;
      nbcol=HR_NBCOL;
      buflen=1024;
      sprintf(path,"%s",im_HR);
    }
    fic=(char *) malloc(100);
    buf=(unsigned char *) malloc(buflen * nblig);
    cn=(unsigned char *) malloc(nblig*nbcol);
  }


  
  sprintf(fic,"%s/%02d%02d/v%02d%03d%02d.bin\0",path,yy,mm,yy,ddd,slot);
  if((fi=fopen(fic,"r"))==0) return 0;
  if(fseek(fi,128,0)!=0) ier++;
  else if(fread(buf,buflen*nblig,1,fi)!=1) ier++;
  if(!ier) {
    for(i=0;i<nblig;i++)
      memcpy(cn+(i*nblig),buf+(i*buflen),nbcol);
  }
  fclose(fi);
  if(ier) {printf("#3 loadminimage: ERROR\n"); exit(0);}
  *cni=cn;


  
  return 1;
}

/*-------------------------------------------------------------------------*/
int min_met(gammasun_noon)
double gammasun_noon;
{
  if(HR){
    if(gammasun_noon*rad2deg < 55.0)
      if(arg_slot_impair_only) return nbdata_min_slotHR[0]-1;
      else return nbdata_min_slotHR[0];
    else
      if(arg_slot_impair_only) return nbdata_min_slotHR[1]-1;
      else return nbdata_min_slotHR[1];
  }
  else {
    if(gammasun_noon*rad2deg < 55.0) return nbdata_min_slotB2[0];
    else                             return nbdata_min_slotB2[1];
  }      
}
/* eof --------------------------------------------------------------------*/

