/*-------------------------------------------------------------------------*/
/*                        ECOLE DES MINES DE PARIS                         */
/*        CENTRE D'ENERGETIQUE - GROUPE TELEDETECTION & MODELISATION       */
/*                       Rue Claude Daunesse, BP 207                       */
/*                   06904 Sophia Antipolis cedex, FRANCE                  */
/*          Tel (+33) 04 93 95 74 49     Fax (+33) 04 93 95 75 35          */
/*                       E-mail : lucien.wald@ensmp.fr                     */
/*-------------------------------------------------------------------------*/
/* map_rhog.c  Lefevre    2002                                             */
/* A software for the construction of an image of ground albedo.           */
/* Ground albedo is the 2nd minimum of the time-series, 2nd in order to    */
/* avoid extremes values, limited to pixels wherein                        */
/* gammasun > 2/3 gammasun_noon. This threshold is itself limited to       */
/* [15:40], gammasun is >15 or >40 or intermediate value according to  */
/* latitude and season. The computed rhog are compared to the reference    */
/* image im_albedo.bin and rhog should be comprised between rhog_ref/2 and */
/* rhog_ref*2.                                                             */
/* rhog is converted in digital number (cn) (*100) between 0 and 253. 255  */
/* denotes non usable pixels. Typical values for reference rhog are 0 to 64*/
/* and typical values for rhog are 0 to 128.                               */
/*-------------------------------------------------------------------------*/
/*Output : gYYMM.bin, where YY is year and MM is month                     */
/*  image in raw format of rhog for a given month                        
  If arg_id display values of 1st and 2nd minima
---------------------------------------------------------------------------
arguments : -d YYYYMM  (year+month)
            [-id pixel identifier from 1 to 118500]
---------------------------------------------------------------------------*/
#include "helio.h"
#include "spatio_time_lib.c"
#include "clearsky_model5_lib.c"
#include "map_lib.c"

void parse_args();
void print_output();

main(argc, argv)
int argc;
char *argv[];
{
  int size = sizeof(double) * B2_NBLIG*B2_NBCOL; /* number of rows * number columns*/
  unsigned char *cn=NULL;
/* malloc is a C function for the dynamical allocation of memory */
  double *Lsat=(double *) malloc(size);/* radiance at satellite level */
  double  Latm;                        /* path radiance */  
  double *rhog0=(double *) malloc(size);/* reference ground albedo */
  double *rhog1=(double *) malloc(size);/* ground albedo = min(rho) month(1st)*/
  double *rhog2=(double *) malloc(size);/* ground albedo = min(rho) month (2nd)*/
  double  rhosat;       /* reflectance satellite*/
  double  rhoatm;       /* reflectance atmosphere */ 
  double  rho;          /* reflectance pixel corrected from atmospheric effects */
  double  lat,lon;      /* latitude  longitude */
  int     lig,col;      /* line, column pixel Meteosat */
  double  z;            /* altitudes (m) */
  double  gammasat;     /* sat. elevation above horizon (radians) */
  double  gammasun;     /* solar elevation angle (radians) */
  double  gammasun_noon;/* idem at noon  */
  double  phisun;       /* solar azimuth angle (radians) */
  double  phisat;       /* satellite azimuth (radians) */
  double  omega_sr;     /* sunrise solar hour angle (radians) */
  double  omega_ss;     /* sunset solar hour angle (radians) */
  double  omega1;       /* UT-0.5 solar hour angle (radians) */
  double  omega2;       /* UT+0.5 solar hour angle (radians) */
  double  delta;        /* declination angle at noon (radians) */
  double  day_angle;    /* */
  double  epsilon;      /* eccentricity de l'orbite */
  double  tst;          /* true solar time */
  int yy,yyyy,mm,dd,ddd;/* year month day julian */
  int slot;             /* well illuminated slots available for B2 format 5h 8h 11h 14h 17h */
  double mu,offset;     /* coeff. calibration */
  unsigned char dark;   /* ...................*/
  double tl;            /* Linke for the current day */
  double *tlam2=(double *) malloc(sizeof(double)*14);  /* TL, Jan to Dec with repetition of extremes for interpolation*/
  int pix,Id;           /* Identifier pixel B2 */
  int i,m,lig1,lig2,col1,col2,ddd1,ddd2;

  B2=1; HR=HRB2=0;
  parse_args(argc,argv);
  sscanf(date,"%04d%02d",&yyyy,&mm);
  
  yy=yyyy % 100;
  ddd1=ddmmyy2ddd(1,mm,yyyy,&ddd1);
  dd=nday[mm]; if(bissextile(yyyy) && mm==2) dd++;
  ddd2=ddmmyy2ddd(dd,mm,yyyy,&ddd2);

  for(i=0; i<B2_NBLIG*B2_NBCOL ; i++) {*(rhog1+i)=*(rhog2+i)=999.0;}
  if(arg_id){
    Get_info_Id(arg_id,&lig1,&col1,NULL,NULL,NULL,NULL);
    lig2=lig1; col2=col1;
  }
  else {
    lig1=1; lig2=B2_NBLIG;
    col1=1; col2=B2_NBCOL;
  }
  
  for(ddd=ddd1 ; ddd<=ddd2 ; ddd++){
    GetCoeff_calib(yyyy,ddd,&dark,&mu,&offset);
    for(slot=1; slot<=NB_SLOT ; slot++){
      if(!loadminimage(yy,mm,ddd,slot,&cn)) continue;
      calibration(cn,dark,mu,offset,Lsat);
      for(lig=lig1; lig<=lig2 ; lig++){
 	for(col=col1 ; col<=col2 ; col++) {
	  if(Get_IdB2(lig,col,&Id,&pix)==0) {*(rhog2+pix)=*(rhog1+pix)=255.0; continue;}/* not usable */
	  Get_info_Id(Id,NULL,NULL,&lat,&lon,tlam2,&z);
	  interpol_TL(ddd,mm,tlam2,&tl);
	  GetParam_day(yyyy,ddd,lat,lon,&day_angle,&delta,&epsilon,&omega_sr,&omega_ss,&gammasat,&phisat,&gammasun_noon);
	  GetParam_hour(slot,lig,lat,lon,day_angle,delta,&gammasun,&omega1,&omega2,&phisun,&tst);
	  if(gammasun*rad2deg < flag_sunoon) continue;
	  calcul_albedo(*(Lsat+pix),epsilon,gammasun,&rhosat);
	  calcul_Latm(z,epsilon,gammasun,gammasat,tl,&Latm);
	  calcul_albedo(Latm,epsilon,gammasun,&rhoatm);
	  calcul_rho_rhoc(z,epsilon,gammasun,gammasat,tl,rhosat,rhoatm,&rho,NULL);
	  *(rhog1+pix)=min(*(rhog1+pix),rho); /* 1er minimum */
	  if(rho<*(rhog2+pix) && rho>*(rhog1+pix)) *(rhog2+pix)=rho; /* 2me minimum */
	  if(comment) getchar();
	}
      }
    }
    if(ddd==ddd1) printf("day "); printf("%3d ",ddd); fflush(stdout);
  }
  Get_rhog(0,0,rhog0);
  if(arg_id) {
    *(rhog1+pix)=max(min(*(rhog1+pix),*(rhog0+pix)*2.0),*(rhog0+pix)/2.0);
    *(rhog2+pix)=max(min(*(rhog2+pix),*(rhog0+pix)*2.0),*(rhog0+pix)/2.0);
    print_output(rhog1+pix,NULL,yy,mm);
    print_output(rhog2+pix,NULL,yy,mm);
  }
  else print_output(rhog2,rhog0,yy,mm);
  printf("#0 end\n");
}

/*-------------------------------------------------------------------------*/
void print_output(rhog,rhog0,yy,mm)
double *rhog,*rhog0;
int yy,mm;
{
  FILE *fo=NULL;
  char *fic=(char *) malloc(50);
  unsigned char *cn=(unsigned char *) malloc(B2_NBLIG*B2_NBCOL);
  double x,xmax=0.0;
  int i;
  
  if(arg_id){
    if(*rhog==255.0) *cn=(unsigned char) 255;/* not usable */
    else             *cn=(unsigned char) min(253,floor(*rhog * convert_rhog + 0.50));
    printf("%02d%02d rhog=%5.3f  cn= %d\n",yy,mm,*rhog,*cn);
    return;
  }
  
  for(i=0 ; i<B2_NBLIG*B2_NBCOL ; i++){
    if(*(rhog+i)==255.0)  *(cn+i)=(unsigned char) 255;/* not usable */
    else { 
      *(rhog+i)=max(min(*(rhog+i),*(rhog0+i)*2.0),*(rhog0+i)/2.0);
      xmax=max(xmax,*(rhog+i));/* for display */
      *(cn+i)=(unsigned char) min(253.0,floor(*(rhog+i) * convert_rhog + 0.50));
    }
  }
  sprintf(fic,"%s/%02d/g%02d%02d.bin",im_cloud_index,yy,yy,mm);
  printf("\n# open output file %s  rhog max=%g cn=%g\n",fic,xmax,floor(xmax*convert_rhog+0.50));
  fo=fopen(fic,"w");
  if(fwrite(cn,1,B2_NBLIG*B2_NBCOL,fo)!=B2_NBLIG*B2_NBCOL) printf("#3 ERROR writing %s\n",fic);
  fclose(fo);}

/*-------------------------------------------------------------------------*/
void parse_args(argc,argv)
int argc;
char *argv[];
{
  int i=1;
  date=(char*) malloc(10);
  while(i<argc) {
    if(strncmp(argv[i],"-c",2)==0) comment=1;
    else if(strncmp(argv[i],"-d",2)==0) sscanf(argv[++i],"%s",date);
    else if(strncmp(argv[i],"-id",3)==0) sscanf(argv[++i],"%d",&arg_id);
    i++;
  }
  if(atoi(date)<198501) {printf("give the month: -d yyyymm\n"); exit(0);}
  printf("\n\n");
}
/* eof --------------------------------------------------------------------*/


