/*-------------------------------------------------------------------------*/
/*                        ECOLE DES MINES DE PARIS                         */
/*        CENTRE D'ENERGETIQUE - GROUPE TELEDETECTION & MODELISATION       */
/*                       Rue Claude Daunesse, BP 207                       */
/*                   06904 Sophia Antipolis cedex, FRANCE                  */
/*          Tel (+33) 04 93 95 74 49     Fax (+33) 04 93 95 75 35          */
/*                       E-mail : lucien.wald@ensmp.fr                     */
/*-------------------------------------------------------------------------*/
/* map_rhog_ref.c  Lefevre    2002                                         */
/* A software for the construction of a reference image of ground albedo   */
/* Ground albedo is the 2nd minimum of the time-series, 2nd in order to    */
/* avoid extremes values, limited to pixels wherein                        */
/* gammasun > 2/3 gammasun_noon. This threshold is itself limited to       */
/* [15:40], gammasun is >15 or >40 or intermediate value according to  */
/* latitude and season. If the pxel is over the land, rhog is > 0.05       */
/* rhog is converted in digital number (cn) (*100) between 0 and 253. 255  */
/* denotes non usable pixels, 254 for missing values (should not occur)    */
/*-------------------------------------------------------------------------*/
/*Output : g0000.bin
  image of reference rhog in raw format
-------------------------------------------------------------------
arguments : [-id n identifier from 1 to 118500] to process only one pixel
---------------------------------------------------------------------------*/
#include "helio.h"
#include "spatio_time_lib.c"
#include "clearsky_model5_lib.c"
#include "map_lib.c"
void parse_args();
void print_output();

main(argc, argv)
int argc;
char *argv[];
{
  int size = sizeof(double) * B2_NBLIG*B2_NBCOL; /* number of rows * number columns*/
  unsigned char *cn=NULL;
/* malloc is a C function for the dynamical allocation of memory */
  double *Lsat=(double *) malloc(size);/* radiance at satellite level */
  double  Latm;                        /* path radiance */  
  double *rhog1=(double *) malloc(size);/* ground albedo = min(rho) month(1st)*/
  double *rhog2=(double *) malloc(size);/* ground albedo = min(rho) month (2nd)*/
  double  rhosat;       /* reflectance satellite*/
  double  rhoatm;       /* reflectance atmosphere */ 
  double  rho;          /* reflectance pixel corrected from atmospheric effects */
  double  lat,lon;      /* latitude  longitude */
  int     lig,col;      /* line, column pixel Meteosat */
  double  z;            /* altitudes (m) */
  double  gammasat;     /* sat. elevation above horizon (radians) */
  double  gammasun;     /* solar elevation angle (radians) */
  double  gammasun_noon;/* idem at noon  */
  double  phisun;       /* solar azimuth angle (radians) */
  double  phisat;       /* satellite azimuth (radians) */
  double  omega_sr;     /* sunrise solar hour angle (radians) */
  double  omega_ss;     /* sunset solar hour angle (radians) */
  double  omega1;       /* UT-0.5 solar hour angle (radians) */
  double  omega2;       /* UT+0.5 solar hour angle (radians) */
  double  delta;        /* declination angle at noon (radians) */
  double  day_angle;    /* */
  double  epsilon;      /* eccentricity de l'orbite */
  double  tst;          /* true solar time */
  int yy,yyyy,mm,dd,ddd;/* year month day julian */
  int slot;             /* well illuminated slots available for B2 format 5h 8h 11h 14h 17h */
  double mu,offset;     /* coeff. calibration */
  unsigned char dark;   /* ...................*/
  double tl;            /* Linke for the current day */
  double *tlam2=(double *) malloc(sizeof(double)*14);  /* TL, Jan to Dec with repetition of extremes for interpolation*/
  int pix,Id;           /* Identifier pixel B2 */
  int i,m,lig1,lig2,col1,col2;

  B2=1; HR=HRB2=0;
  parse_args(argc,argv);
  for(i=0; i<B2_NBLIG*B2_NBCOL ; i++) {*(rhog1+i)=*(rhog2+i)=999.0;}
  if(arg_id){
    Get_info_Id(arg_id,&lig1,&col1,NULL,NULL,NULL,NULL);
    lig2=lig1; col2=col1;
  }
  else {
    lig1=1; lig2=B2_NBLIG;
    col1=1; col2=B2_NBCOL;
  }

  for(yyyy=1985 ; yyyy<=1997 ; yy++){ 
    yy=yyyy % 100;
    for(mm=1 ; mm<=7 ; mm+=6){
      for(dd=1 ; dd<32 ; dd++){
	ddmmyy2ddd(dd,mm,yyyy,&ddd);
	if(yyddd2mm(yyyy,ddd,&m)!=mm) break;
	GetCoeff_calib(yyyy,ddd,&dark,&mu,&offset);
	printf("# %02d%02d%3d\n",yy,mm,dd);
	for(slot=1; slot<49 ; slot++){
	  if(!loadminimage(yy,mm,ddd,slot,&cn)) continue;
	  calibration(cn,dark,mu,offset,Lsat);    
	  for(lig=lig1; lig<=lig2 ; lig++){
	    for(col=col1 ; col<=col2 ; col++) {
	      if(Get_IdB2(lig,col,&Id,&pix)==0) {*(rhog2+pix)=*(rhog1+pix)=255.0; continue;}/* not usable */
	      Get_info_Id(Id,NULL,NULL,&lat,&lon,tlam2,&z);
	      interpol_TL(ddd,mm,tlam2,&tl);
	      GetParam_day(yyyy,ddd,lat,lon,&day_angle,&delta,&epsilon,&omega_sr,&omega_ss,&gammasat,&phisat,&gammasun_noon);
	      GetParam_hour(slot,lig,lat,lon,day_angle,delta,&gammasun,&omega1,&omega2,&phisun,&tst);
	      if(gammasun*rad2deg < flag_sunoon) continue;
	      calcul_albedo(*(Lsat+pix),epsilon,gammasun,&rhosat);
	      calcul_Latm(z,epsilon,gammasun,gammasat,tl,&Latm);
	      calcul_albedo(Latm,epsilon,gammasun,&rhoatm);
	      calcul_rho_rhoc(z,epsilon,gammasun,gammasat,tl,rhosat,rhoatm,&rho,NULL);
	      *(rhog1+pix)=min(*(rhog1+pix),rho); /* 1st minimum */
	      if(rho<*(rhog2+pix) && rho>*(rhog1+pix)) *(rhog2+pix)=rho; /* 2nd minimum */
	      if(z>0){
		*(rhog1+pix)=max(*(rhog1+pix),0.05);
		*(rhog2+pix)=max(*(rhog2+pix),0.05);
	      }
	      if(comment) getchar();
	    }
	  }
	}
      }
    }
  }
  if(arg_id) {
    print_output(rhog1+pix);
    print_output(rhog2+pix);
  }
  else {
    print_output(rhog2);
  }
}

/*-------------------------------------------------------------------------*/
void print_output(rhog)
double *rhog;
{
  FILE *fo=NULL;
  char *fic=(char *) malloc(50);
  unsigned char *cn=(unsigned char *) malloc(B2_NBLIG*B2_NBCOL);
  double x,xmax=0.0;
  int i;
  int     lig,col;      /* lig col pixel Meteosat */
  double  lat,lon,z;    /* altitudes (m) */
  double *tlam2=(double *) malloc(sizeof(double)*14);  /* TL*/
  int pix,Id;           /* Identifier pixel B2 */ 

  if(arg_id){
    if(*rhog==255.0)      *cn=(unsigned char) 255;/* not usable */
    else if(*rhog==999.0) *cn=(unsigned char) 254;/* should not happen */
    else                  *cn=(unsigned char) min(253,floor(*rhog * convert_rhog + 0.50));
    printf("rhog=%5.3f  cn= %d\n",*rhog,*cn);
    return;
  }
  
  for(i=0 ; i<B2_NBLIG*B2_NBCOL ; i++){
    if(*(rhog+i)==255.0)  *(cn+i)=(unsigned char) 255;/* not usable */
    else if(*(rhog+i)==999.0) *(cn+i)=(unsigned char) 254;/* should not happen */
    else {
      xmax=max(xmax,*(rhog+i));
      *(cn+i)=(unsigned char) min(253.0,floor(*(rhog+i) * convert_rhog + 0.50));
    }
  }
  for(lig=1 ; lig<=B2_NBLIG; lig++){
    for(col=1 ; col<=B2_NBCOL ; col++) {
      if(Get_IdB2(lig,col,&Id,&pix)==0) continue;
      Get_info_Id(Id,NULL,NULL,&lat,&lon,tlam2,&z);
      if(z>0 && *(cn+pix)==0){       /* replaces 0 values for land pixels by 5 */
	*(cn+pix)=5; /* rho=0.05 * 100 */
      }
    }
  }
  sprintf(fic,"g0000.bin");
  printf("\n# open output file %s  rhog max=%g cn=%g\n",fic,xmax,floor(xmax*convert_rhog+0.50));
  fo=fopen(fic,"w");
  if(fwrite(cn,1,B2_NBLIG*B2_NBCOL,fo)!=B2_NBLIG*B2_NBCOL) printf("#3 ERROR writing %s\n",fic);
  fclose(fo);
  system("info_im_albedo g0000.bin"); /* to control if missing values, and display min and max */
  printf("#0 end\n");
}

/*-------------------------------------------------------------------------*/
void parse_args(argc,argv)
int argc;
char *argv[];
{
  int i=1;
  while(i<argc) {
    if(strncmp(argv[i],"-c",2)==0) comment=1;
    else if(strncmp(argv[i],"-id",3)==0) sscanf(argv[++i],"%d",&arg_id);
    i++;
  }
  printf("\n\n");
}
/* eof --------------------------------------------------------------------*/


