/*-------------------------------------------------------------------------*/
/*                        ECOLE DES MINES DE PARIS                         */
/*        CENTRE D'ENERGETIQUE - GROUPE TELEDETECTION & MODELISATION       */
/*                       Rue Claude Daunesse, BP 207                       */
/*                   06904 Sophia Antipolis cedex, FRANCE                  */
/*          Tel (+33) 04 93 95 74 49     Fax (+33) 04 93 95 75 35          */
/*                       E-mail : lucien.wald@ensmp.fr                     */
/*-------------------------------------------------------------------------*/
/* spatio_time_lib.c   M. Lefevre 2002                                     */
/* Contains functions for getting information about sun geometry, Linke    */
/* turbidity factor, elevation ... for a given pixel. Includes also        */
/* functions about leap year and julian days and spatial interpolation     */
/*-------------------------------------------------------------------------*/

#include "meteosatlib.c"
#include "solar_geometry.c"
int  Get_IdB2();
void Get_info_Id();
void interpol_TL();
void GetParam_day();
void GetParam_hour();
int  ppv();
void GetTerrainBase();
int  bissextile();
int  yyddd2mm();
int  ddmmyy2ddd();
FILE *OpenFile();

/*------------------------------------------------------------------------
Reading the image of pixels B2 identifiers
input: lig (row) col starting from 1
output: id=0 pixel non usable: not in Earth disk or gammasat<15
        id=n identifier [1:118500] of usable pixels starting from NW 
        pix starting from 0 (pixel number)
*/
int Get_IdB2(lig,col,Id,pix)
int lig,col,*Id,*pix;
{
  static FILE *fi=0;
  int ier=0;

  *Id=0;
  if(fi==0) if((fi=OpenFile(im_IdB2))==0) exit(0);
 
  *pix=(lig-1)*B2_NBCOL+col-1; /* lig col from 1, pix from 0 */
  if(*pix<0 || *pix>B2_NBLIG*B2_NBCOL) ier++;
  if(ier==0) if(fseek(fi,*pix*4,0)!=0) ier++;
  if(ier==0) if(fread(Id,4,1,fi)!=1) ier++;
  if(ier==0 && web==1) *Id=swapint(*Id);
  if(comment) printf("# Get_IdB2: lig.%d col.%d pix.%d Id.%d\n",lig,col,1+*pix,*Id);
  return *Id;
}
/*---------------------------------------------------------------------------
  Reading the database of the identifiers, getting elevation and TL
  z= Terrain Base 5' ocean=0 land below sea level=1 land>1 with step = 1m
  TL= images 5' coded char*20 cf. SoDa */

void Get_info_Id(Id,lig,col,lat,lon,tl,z)
int Id,*lig,*col; /* from 1 */
double *lat,*lon; /* degrees */
double *tl,*z;
{
  static FILE *fi=0;
  const int size=30;
  int Id0,lat0,lon0;
  short lig0,col0,z0,x;
  static unsigned char *tl0;
  int ier=0,m;

  if(fi==0){
    if((fi=OpenFile(tab_IdB2))==0) exit(0);
    tl0=(unsigned char *) malloc(13);
  }
  if(fseek(fi,(Id-1)*size,0)!=0) ier++;
  if(fread(&Id0,4,1,fi)!=1) ier++;  if(ier==0 && web==1) Id0=swapint(Id0);
  if(fread(&lig0,2,1,fi)!=1) ier++; if(ier==0 && web==1) {swab(&lig0,&x,2); lig0=x;}
  if(fread(&col0,2,1,fi)!=1) ier++; if(ier==0 && web==1) {swab(&col0,&x,2); col0=x;}
  if(fread(&lat0,4,1,fi)!=1) ier++; if(ier==0 && web==1) lat0=swapint(lat0);
  if(fread(&lon0,4,1,fi)!=1) ier++; if(ier==0 && web==1) lon0=swapint(lon0);
  for(m=1;m<13;m++) if(fread(tl0+m,1,1,fi)!=1) ier++;
  if(fread(&z0,2,1,fi)!=1) ier++; if(ier==0 && web==1) {swab(&z0,&x,2); z0=x;}
  
  if(lig) *lig=lig0;
  if(col) *col=col0;
  if(lat) *lat=(double)lat0/100.0;
  if(lon) *lon=(double)lon0/100.0;
  if(z)   *z=(double)z0; /* terrain base 5' */
  if(tl){
    for(m=1;m<13;m++) *(tl+m)=(double)*(tl0+m)/20.0; /* tl*20 in image 5' */
    *(tl+0)=*(tl+12);/* Dec is repeated before Jan for interpolation */
    *(tl+13)=*(tl+1);/* Jan ............after Dec .................. */
  }
  if(comment || ier || Id!=Id0) {
    printf("# Get_info_Id.%d: ",Id);
    if(lig && col) printf("lig=%d col=%d (from 1) ",*lig,*col);
    if(lat) printf("lat=%g ",*lat);
    if(lon) printf("lon=%g ",*lon);
    if(z)   printf("z=%gm ",*z);
    if(tl) {printf("tl[mm] "); for(m=1;m<=12;m++) printf("%3.1f ",*(tl+m));}
    printf("\n");
    if(ier){printf("#3 Get_info_Id: ERROR\n#0\n");exit(0);}
  }
}

/*---------------------------------------------------------------------------
  Interpolation of TL for a given day, using two monthly values */

void interpol_TL(ddd,mm,tlam2,tl)
int ddd,mm;
double *tlam2,*tl;
{
  float w1,w2;
  int dd;
  dd=ddd-Sday[mm-1];
  w2=abs(15-dd)/30.0; w1=1.0-w2;
  *tl= w1 * *(tlam2+mm) + w2 * (dd < 15 ? *(tlam2+mm-1):*(tlam2+mm+1));
  if(comment) printf("# Interpolation TL: %02d/%02d tl=%3.1f w1=%5.3f w2=%5.3f\n",dd,mm,*tl,w1,w2);
}

/*-------------------------------------------------------------------------*/
void GetParam_day(yyyy,ddd,lat,lon,day_angle,delta,epsilon,omega_sr,omega_ss,gammasat,phisat,gammasun_noon)
int yyyy,ddd;
double lat,lon; /* degrs */
double *day_angle;
double *delta;      /* solar declination angle at noon (in radians) */
double *epsilon;    /* eccentricity */
double *omega_sr,*omega_ss;
double *gammasat,*phisat;/* elevation and azimuth of satellite */
double *gammasun_noon;/* elevation of sun at local noon */
{
  double gamma_riset=0.0;
  double omega,thetasun;
  int ier=0;
  *day_angle=0;
  *epsilon=0;
  ier+=Day_Angle(ddd,day_angle);
  ier+=corr_distance(*day_angle,epsilon);
  ier+=declination_sun(yyyy,ddd,lon*deg2rad,delta);
  ier+=sunrise_hour_angle(lat*deg2rad,*delta,gamma_riset,omega_sr,omega_ss);
  ier+=satazim(lat,lon,phisat);
  ier+=satelev(lat,lon,gamma_riset,gammasat);
  ier+=solar_hour_angle(12.0, &omega);
  ier+=elevation_zenith_sun(lat*deg2rad,*delta,omega,gammasun_noon,&thetasun);
  flag_sunoon= min(max(flag_sun,*gammasun_noon*rad2deg*2.0/3.0),40);
  if(ier || comment){
    printf("# GetParam_day %3d: delta=%5.1f epsilon=%5.2f \n",ddd,*delta,*epsilon);
    printf("#                : gammasun_noon=%5.2frd %5.3f\n",*gammasun_noon,*gammasun_noon*rad2deg);
    printf("#                : omega_sr=%5.2f omega_ss=%5.2f\n",*omega_sr*rad2deg,*omega_ss*rad2deg);
    printf("#                : gammasat=%6.2f phisat=%6.2f\n",*gammasat,*phisat);
    printf("#                : flag_sunoon=%6.2f\n",flag_sunoon);
    if(ier) printf("#3 GetParam_day ERROR\n");
    getchar();
  }
}

/*---------------------------------------------------------------------------
   UT[slot] = hour (in UTC) of the beginning of acquisition (South line)
   acquisition time of a pixel is UT[0]=UT[slot] + time lag for the line under 
   concern = 25'/2500 lines = 0.6"/line */

void GetParam_hour(slot,lig,lat,lon,day_angle,delta,gammasun,omega1,omega2,phisun,tst)
int slot,lig; /* lines starting from 1 */
double lat,lon; /* degrees */
double day_angle,delta;
double *gammasun;      /* solar elevation angle (radians) */
double *phisun;        /* solar azimuth (radians) */
double *omega1,*omega2;
double *tst;           /* True solar time */
{
  double LAT;      /* local apparent time = TST (true solar time) */
  double omega;    /* solar hour angle (in radians) */
  double thetasun; /* solar zenithal angle (in radians) */
  double ligne;
  int ier=0;
  if(lig==0) ier++;
  ligne=(double) lig;
  UT[0]= UT[slot] + (2496.0-ligne*6)*0.6/3600;/* line from 1 */
  ier+=UT_to_LAT(UT[0],day_angle,lon*deg2rad,tst);  /* decimal hour */
  ier+=solar_hour_angle(*tst, &omega);
  ier+=elevation_zenith_sun(lat*deg2rad,delta,omega,gammasun,&thetasun);
  ier+=azimuth_sun(lat*deg2rad,delta,omega,*gammasun,phisun);
  ier+=UT_to_LAT(max(UT[0]-0.50,0), day_angle, lon*deg2rad, &LAT);
  ier+=solar_hour_angle(LAT, omega1);
  ier+=UT_to_LAT(min(UT[0]+0.50,24.0), day_angle, lon*deg2rad, &LAT);
  ier+=solar_hour_angle(LAT, omega2);

  if(comment || ier) {
    printf("# GetParam_hour: UT[slot %d]=%gh UT[0]=%gh local time=%5.1fh\n",slot,UT[slot],UT[0],*tst);
    printf("#              : lat=%4.2f lon=%4.2f  ligne=%6.0f ",lat,lon,ligne);
    printf("\n#             : phisun    %6.2frd %7.1f ",*phisun,*phisun*rad2deg);
    printf("\n#             : gammasun  %6.2frd %7.1f ",*gammasun,*gammasun*rad2deg);
    printf("\n#             : thetasun  %6.2frd %7.1f ",thetasun,thetasun*rad2deg);
    printf("\n#             : omega1    %6.2frd %7.1f ",*omega1,*omega1*rad2deg);
    printf("\n#             : omega2    %6.2frd %7.1f\n",*omega2,*omega2*rad2deg);
    if (ier) {printf("#3 Get_Param_hour: ERROR %d\n",ier); exit(0);}
  }
}
/*-------------------------------------------------------------------------*/
/*
Searching the NBPIX closest neighbouring pixels by the gravity method:
w(i)=1/distance. The distance is either geodetic, either effective between these
pixels and the geographical point (lat,lon).
cf. Lefvre M., Remund J., Albuisson M., Wald L., Study of effective distances for
interpolation schemes in meteorology. European Geophysical Society, 27th General
Assembly, Geophysical Research Abstracts, vol. 4, April 2002, EGS02-A-03429.
                       d_eff = f_NS(d_go + d_oro)
Returns the number of usables pixels. The vector Id contains the NBPIX ranked by weight,
the closest being in Id+0.
*/
int ppv(lat0,lon0,z0,Id0,W0,distance)
double lat0,lon0,*z0;
int *Id0;
double *W0;
int distance;/* 1=effective distance 0=distance go. */
{
  int nbpix=0,i,i0,i1,j,lig,col,pix;
  int v=16;/* HRb2v returns a vector of 16 pixels around the point */
  static int *id=NULL;
  static double *w=NULL;
  static int *l=NULL;
  static int *c=NULL;
  static int *u=NULL;
  static float *D=NULL;
  double lat,lon,z,wW,d_oro,d_geo,d_eff,f_NS;
  double *tlam2=(double *) malloc(sizeof(double)*14);  /* TLinke per month */

  for(i=0 ; i<NBPIX ; i++) {*(Id0+i)=0; *(W0+i)=0.0;}
  if(l==NULL) {
    id=(int*) malloc(sizeof(int)*v);
    w=(double*) malloc(sizeof(double)*v);
    l=(int*) malloc(sizeof(int)*v);
    c=(int*) malloc(sizeof(int)*v);
    u=(int*) malloc(sizeof(int)*v);
    D=(float*) malloc(sizeof(float)*v);
  }
  if(*z0<0) {
    GetTerrainBase(lat0,lon0,z0,u);
    if(*u==0) return 0;/* lat lon non usable */
  }
  latlon_to_HRb2v(lat0,lon0,l,c,u,D); /* pixel 0 = the closest coord. lig. col. */
  if(comment) {
    printf("# lat=%5.2f lon=%5.2f z=%-5.0f   closest neighbours in distance ",lat0,lon0,*z0);
    if(distance==1) printf("effective:\n"); else printf("geodetic:\n");
  }
  if(*u==0) return 0;/* 1st pixel non usable */
  for (i=0 ; i<v ; i++){
    *(id+i)=0; *(w+i)=0;
    HRb2_to_B2(*(l+i), *(c+i), &lig, &col);
    if(Get_IdB2(lig,col,id+i,&pix)==0) continue;
    nbpix++;
    Get_info_Id(*(id+i),&lig,&col,&lat,&lon,tlam2,&z); 
    d_geo= R * acos( (sin(lat0*deg2rad)*sin(lat*deg2rad)) + (cos(lat0*deg2rad)*cos(lat*deg2rad)*cos(lon0*deg2rad-lon*deg2rad)));
    if(distance==1){
      d_oro=500.0 * (*z0-z) * 0.0010;
      f_NS= 1.0+0.30*fabs(lat0-lat)*(1.0+0.50*(sin(lat0*deg2rad)+sin(lat*deg2rad)));
      d_eff=(pow(d_geo,2.0) + pow(d_oro,2.0)) * pow(f_NS,2.0);
      d_eff=sqrt(d_eff);
      *(w+i)=(1.0 / (d_eff==0.0 ? 0.01 : d_eff));
    }
    else *(w+i)=(1.0/ (d_geo==0.0 ? 0.01 : d_geo));
  }
  nbpix=0; wW=0;
  for(j=0 ; j<v ; j++) {
    if(distance==1){/* ranking decreasing in effective distance */
      for(i=j+1 ; i<v ; i++){ 
	if(*(w+i) > *(w+j))
	  {*u=*(id+j); *(id+j)=*(id+i); *(id+i)=*u; 
	   *D=(float)*(w+j);*(w+j)=*(w+i);*(w+i)=(double)*D;}
      }
    }
    if(*(id+j)==0) continue;
    *(Id0+nbpix)=*(id+j);
    *(W0+nbpix)=*(w+j);
    wW+=*(w+j);
    if(comment) printf("# ppv :Id[%d]=%d dist=%10.8f\n",nbpix,*(Id0+nbpix),1/ *(W0+nbpix));
    nbpix++;
    if(nbpix==NBPIX) break;
  }
  for(i=0 ; i<nbpix ; i++) *(W0+i)/= wW;
  for(i=0,wW=0.0 ; i<nbpix ; i++) {
    wW+=*(W0+i);/* control sum of weights =1 */
    if(comment) printf("# ppv :Id[%d]=%d w=%5.3f Ww=%5.3f\n",i,*(Id0+i),*(W0+i),wW);
  }
  if(nbpix == NBPIX && (float)wW!=1.0) {printf("#3 ppv: ERROR sum of weights %g != 1\n",wW); exit(0);}
  return nbpix; /* nb of pixels usables */
}

/*-------------------------------------------------------------------------*/
/*  orographie= deerived from Terrain Base 5' +90N-90S  -180W+180E
**               ocean=0 land below sea level=1 land>1 with step = 1m
**  minimage /v2/ma/tbase5.hdr  canal 102: /v2/ma/tbas_17.bin
**  image coded on 2 bytes inverted (short int, 16 bits)
*/
void GetTerrainBase(lat,lon,z,usable)
double lat,lon,*z; /* lat lon en degrees */
int *usable;
{
  int i,ier=0,lig,col;
  double nbpix_par_deg = 12.0; /* resolution 5' = 1/12 */
  int nblig=2160;              /* latitude  +N -S */ 
  int nbcol=4320;              /* longitude -W +E */
  short int *tb5=(short int *) malloc(sizeof(short int)*2);
  char *file=(char *) malloc(50);
  FILE *fi=0;

  *z=-999.0; *usable=0; ier=0;
  lig=(nblig/2- (lat<0 ? floor(lat*nbpix_par_deg-0.50) : ceil(lat*nbpix_par_deg-0.50)));
  col=nbcol/2+ceil(lon*nbpix_par_deg-0.50);
  if(lig<1 || lig>nblig) ier++;
  if(col<1 || col>nbcol) ier++;
  sprintf(file,"%s/tbas_17.bin",im_TB5);
  if((fi=OpenFile(file))==0) exit(0);
  i=128+2*((lig-1)*nbcol+col-1);
  if(fseek(fi,i,0)!=0) ier++;
  if(fread(tb5,2,1,fi)!=1) ier++;
  swab(tb5,tb5+1,2); /* inversion des octets */
  if(ier==0) {*z=(double) *(tb5+1); *usable=1;}
  fclose(fi);
  if(comment) printf("# TerrainBase: lat %g lon %g lig %d col %d  z=%5.0f m usable=%d\n",lat,lon,lig,col,*z,*usable);
}

/*---------------------------------------------------------------------------
 */
int bissextile(yyyy) /* Is the year yyyy a leap year? */
int yyyy;
{
  if( ((yyyy % 4)==0 && (yyyy % 100)!=0) || (yyyy % 400)==0) return 1;
  return 0;
}
/*---------------------------------------------------------------------------
  Compute month from julian day */
int yyddd2mm(yyyy,ddd,mm)
int yyyy,ddd,*mm;
{
  if(bissextile(yyyy)) for(*mm=1; *mm < 13 && Sdayb[*mm]< ddd ; *mm+=1);
  else                 for(*mm=1; *mm < 13 && Sday[*mm]< ddd ; *mm+=1);
  if(*mm>12) *mm=0;
  return *mm;
}
/*---------------------------------------------------------------------------
  Compute julian day from day, month, year */
int ddmmyy2ddd(dd,mm,yyyy,ddd)
int dd,mm,yyyy,*ddd;
{
  if(yyyy < 80) yyyy+=2000;  else if(yyyy < 1900) yyyy+=1900;
  if(bissextile(yyyy)) {*ddd = dd + Sdayb[mm-1];if(*ddd<=Sdayb[mm-1] || *ddd>Sdayb[mm]) *ddd=0;}
  else                 {*ddd = dd + Sday[mm-1]; if(*ddd<=Sday[mm-1] || *ddd>Sday[mm]) *ddd=0;}
  return *ddd;
}
/*---------------------------------------------------------------------------
  Open a file and returns a pointer on bof or NULL if file is absent */

FILE *OpenFile(name)
char *name;
{
  FILE *fi=0;
  fi=fopen(name,"r");
  if(fi==0) printf("\r#2 no file %s\n#0\n",name);
  else if(comment) printf("\r# open %s\n",name);
  return fi;
}
/* eof --------------------------------------------------------------------*/

